var cheerio = require('cheerio');
var stream = require('stream');

exports.IQDBLibs_2D = void 0;
(function (IQDBLibs_2D) {
  IQDBLibs_2D[IQDBLibs_2D["danbooru"] = 1] = "danbooru";
  IQDBLibs_2D[IQDBLibs_2D["konachan"] = 2] = "konachan";
  IQDBLibs_2D[IQDBLibs_2D["yande.re"] = 3] = "yande.re";
  IQDBLibs_2D[IQDBLibs_2D["gelbooru"] = 4] = "gelbooru";
  IQDBLibs_2D[IQDBLibs_2D["sankaku channel"] = 5] = "sankaku channel";
  IQDBLibs_2D[IQDBLibs_2D["e-shuushuu"] = 6] = "e-shuushuu";
  IQDBLibs_2D[IQDBLibs_2D["zerochan"] = 11] = "zerochan";
  IQDBLibs_2D[IQDBLibs_2D["anime-picture"] = 13] = "anime-picture";
})(exports.IQDBLibs_2D || (exports.IQDBLibs_2D = {}));
exports.IQDBLibs_3D = void 0;
(function (IQDBLibs_3D) {
  IQDBLibs_3D[IQDBLibs_3D["3dbooru"] = 7] = "3dbooru";
  IQDBLibs_3D[IQDBLibs_3D["idol"] = 9] = "idol";
})(exports.IQDBLibs_3D || (exports.IQDBLibs_3D = {}));

function parseSimilarity(txt) {
  const result = txt.match(/(-?\d+\.?\d*)(%?)/);
  if (result) {
    return result[2] == undefined || result[2] == '' ? parseFloat(result[1]) : parseFloat(result[1]) / 100;
  } else {
    return null; //couldn't parse similarity
  }
}
function parseSizeAndType(txt) {
  const result = txt.match(/(\d+)×(\d+)(?: \[([A-z]*)\])?/);
  if (result) {
    return {
      size: {
        width: parseInt(result[1]),
        height: parseInt(result[2])
      },
      type: result[3]
    };
  } else {
    return txt;
  }
}
const availCharset = '0123456789abcdefghijklmnopqrstuvwxyz@ABCDEFGHIJKLMNOPQRSTUVWXYZ_';
const availCharsetLen = availCharset.length;
function getRandomName() {
  const length = 5 + (Math.random() * 10 | 0);
  let str = '';
  for (let i = 0; i < length; i++) {
    str += availCharset[Math.random() * availCharsetLen | 0];
  }
  return str + '.jpg';
}
async function asyncIterableToArray(asyncIterable) {
  const result = [];
  for await (const item of asyncIterable) {
    result.push(item);
  }
  return result;
}

const defaultConfig = {
  baseDomain: 'iqdb.org',
  similarityPass: 0.6,
  userAgent: 'node'
};
function _addToForm(form, libs) {
  for (const lib of libs) {
    form.append('service[]', lib.toString());
  }
}
/**
 *
 * @param body 服务器返回的body
 * @param noSource 指示结果中是否应该有source字段
 * @returns
 */
function parseResult(body, similarityPass, noSource) {
  const $ = cheerio.load(body);
  let ok = false;
  const err = $('.err');
  if (err.length > 0) {
    return {
      ok: false,
      err: err.text()
    };
  }
  const service = $('input[type=checkbox][name="service[]"][checked]').toArray().map(element => {
    const value = parseInt(element.attribs.value);
    const name = exports.IQDBLibs_2D[value] || exports.IQDBLibs_3D[value];
    if (!name) console.warn('Unknown lib: ' + value);
    return value;
  });
  const data = $('#pages').children('div').toArray().map(page => {
    const rows = $(page).find('tr');
    const head = $(rows[0]).text();
    if (head == 'Your image') {
      const sizeAndType = parseSizeAndType($(rows[3]).text());
      return {
        head,
        img: $(rows[1]).find('img').attr('src'),
        name: $(rows[2]).text(),
        // no similarity for your own image
        similarity: null,
        ...(typeof sizeAndType == 'object' ? sizeAndType : {
          sizeAndType
        })
      };
    } else if (head == 'No relevant matches') {
      return;
    } else {
      let similarity, sizeAndType, source;
      if (noSource) {
        similarity = parseSimilarity($(rows[3]).text());
        sizeAndType = parseSizeAndType($(rows[2]).text());
      } else {
        similarity = parseSimilarity($(rows[4]).text());
        sizeAndType = parseSizeAndType($(rows[3]).text());
        source = $(rows[2]).text().split(' ');
      }
      if (similarity >= similarityPass) ok = true;
      const imgBox = $(rows[1]).find('a');
      return {
        head,
        sourceUrl: imgBox.attr('href'),
        similarity,
        img: imgBox.find('img').attr('src'),
        ...(typeof sizeAndType == 'object' ? sizeAndType : {
          sizeAndType
        }),
        source
      };
    }
  }).filter(item => item != undefined);
  return {
    ok,
    data,
    service
  };
}
function makeSearchFunc(config) {
  return async function searchPic(pic, {
    lib,
    forcegray,
    service: libs,
    fileName
  }) {
    const isMultiLib = lib == 'www' || lib == '3d';
    const form = new FormData();
    if (typeof pic == 'string') {
      form.append('url', pic);
    } else if (pic instanceof Buffer) {
      form.append('file', new Blob([pic]), fileName || getRandomName());
    } else if (pic instanceof stream.Readable) {
      form.append('file', new Blob(await asyncIterableToArray(pic)), fileName || getRandomName());
    } else throw new TypeError('expect string | Buffer | Readable');
    if (isMultiLib && libs) _addToForm(form, libs);
    if (forcegray) form.append('forcegray', 'true');
    const resp = await fetch(`https://${lib}.${config.baseDomain}`, {
      method: 'POST',
      body: form,
      headers: {
        'User-Agent': config.userAgent
      },
      ...config.fetchOptions
    });
    if (resp.ok) {
      return parseResult(await resp.text(), config.similarityPass, !isMultiLib);
    } else {
      return {
        ok: false,
        err: 'HTTP ' + resp.status
      };
    }
  };
}
const searchPic = makeSearchFunc(defaultConfig);

exports.defaultConfig = defaultConfig;
exports.makeSearchFunc = makeSearchFunc;
exports.searchPic = searchPic;
//# sourceMappingURL=index.js.map
